@@ -6,13 +6,15 @@
 
 import collections, urlparse
 
-from learnsprout.errors import error_from_json
+from learnsprout import errors
 
 class Resource(object):
   """
     **Resource** is a base class for all resources available on the REST API.
   """
   uri_resource_name = "uri_resource_name"
+  has_deltas = False
+
   def __init__(self, owning_client, base_uri):
     self.base_uri = base_uri
     self.owning_client = owning_client
@@ -43,15 +45,21 @@ class ResourceInstance(Resource):
     for subresource_class in self.subresources:
       subresource = subresource_class(self.owning_client, self.uri)
       instantiated.append(subresource)
-      setattr(self, subresource_class.collection_name, subresource)
+      setattr(self, subresource.get_collection_name(), subresource)
+      if subresource_class.has_deltas:
+        delta_companion_resource = ResourceDeltaCollection(self.owning_client, subresource.uri)
+        setattr(subresource, delta_companion_resource.get_collection_name(), delta_companion_resource)
     return instantiated
 
-  def populate_from_json(self, json_obj):
+  def populate_from_json(self, json_obj, id_attr="id"):
     """
       Build a **ResourceInstance** that has member properties matching
       the JSON keys.
     """
-    json_obj["ls_id"] = json_obj["id"]
+    try:
+      json_obj["ls_id"] = json_obj[id_attr]
+    except KeyError, e:
+      raise errors.BadRecord('Cannot create record from JSON %r. Missing key %r' % (json_obj, e.message))
     del json_obj["id"]
     self.__dict__.update(json_obj)
 
@@ -60,13 +68,16 @@ class ResourceInstance(Resource):
     return "%s/%s" % (self.base_uri, self.ls_id)
 
   # a string template for __str__
-  str_template = "%(type)s: %(name)s, id: %(id)s"
+  str_template = "%(type)s(name='%(name)s', id='%(id)s')"
+
+  def __repr__(self):
+    return str(self)
 
   def __str__(self):
     d = self.__dict__.copy()
     d.update({
         'type': self.__class__.__name__,
-        'name': getattr(self, 'name', None),
+        'name': getattr(self, 'uri_resource_name', None),
         'id': getattr(self, 'ls_id', None)
         })
     return self.str_template % d
@@ -83,6 +94,8 @@ class ResourceCollection(Resource):
   collection_name = "collection_name"
   # The class of the instances that live off this collection
   instance_class = ResourceInstance
+  # has accompanying delta routes for data change logs
+  has_deltas = True
 
   def __init__(self, owning_client, base_uri):
     super(ResourceCollection, self).__init__(owning_client, base_uri)
@@ -90,6 +103,10 @@ class ResourceCollection(Resource):
     self.offset = None
     self.next = None
 
+  def get_collection_name(self):
+    """ By default just return the class attribute, but subclasses may override this behavior """
+    return self.collection_name
+
   def instance_from_json(self, json_obj):
     """ Create an appropriate **ResourceInstance** from a JSON object """
     instance = self.instance_class(self.owning_client, self.uri)
@@ -98,7 +115,7 @@ class ResourceCollection(Resource):
     return instance
 
   def get(self, ls_id):
-    """ Get a specific **ResourceInstance** as identified by it's LearnSprout ID"""
+    """ Get a specific **ResourceInstance** as identified by its LearnSprout ID"""
     uri = "%s/%s" % (self.uri, ls_id)
     response = self.owning_client.make_request(uri)
     result = self.instance_from_json(response)
@@ -116,12 +133,50 @@ class ResourceCollection(Resource):
       params["since"] = since
     return IterableResult(self.owning_client, self.instance_from_json, self.uri, params)
 
+  def __repr__(self):
+    return "%s('%s')" % (self.__class__.__name__, self.get_collection_name())
+
+class ResourceDelta(ResourceInstance):
+  subresources = []
+  has_deltas = False
+
+  @property
+  def uri_resource_name(self):
+    return '%s.delta' % self.base_uri.split('/')[-1]
+
+class ResourceDeltaCollection(ResourceCollection):
+  collection_name = "deltas"
+  instance_class = ResourceDelta
+  has_deltas = False
+
+  @property
+  def uri_resource_name(self):
+    return '%s.delta' % self.base_uri.split('/')[-1]
+
+  @property
+  def uri(self):
+    """
+    Creates a /delta/* uri for the base resource collection
+    Unlike standard resource collections, delta resources have a prefix instead of a suffix in the uri path element
+    """
+    uri_parts_dict = urlparse.urlparse(self.base_uri)._asdict()
+    # modify resource's uri with a '/delta' prefix
+    uri_parts_dict['path'] = '/delta%(path)s' % uri_parts_dict
+    delta_uri = urlparse.ParseResult(**uri_parts_dict).geturl()
+    return delta_uri
+
+  def get(self, ls_id):
+    raise errors.UnsupportedRoute('ResourceDelta objects cannot be retrieved by its LearnSprout ID')
+
+  def __repr__(self):
+    return "%s('%s.%s')" % (self.__class__.__name__, self.base_uri.split('/')[-1], self.get_collection_name())
+
 class IterableResult(object):
   """
     An **IterableResult** is a generator which can be iterated over to grab
     **ResourceInstances** from a **ResourceCollection**.  It abstracts away the paging
     support in the LearnSprout API and will automatically pull new results when a page
-    has been exhaused.
+    has been exhausted.
   """
   def __init__(self, owning_client, instance_to_class_func, uri, params):
     self.client = owning_client
@@ -151,7 +206,7 @@ class IterableResult(object):
 
       response = self.client.make_request(self.uri, parameters)
       if "status_code" in response and response["status_code"] >= 400:
-        raise error_from_json(response)
+        raise errors.error_from_json(response)
 
       if "data" in response and "next" in response: # Paginated collection
         if response["next"]:
@@ -174,7 +229,11 @@ class IterableResult(object):
       raise StopIteration()
 
     # i haz resultz, carry on...
-    result = self.__buffer.popleft()
+    try:
+      result = self.__buffer.popleft()
+    except IndexError:
+      raise StopIteration()
+
     if not self.__buffer and not self.__next:
       self.__done = True
     return result
